﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NDesk.DBus;

namespace org.webosinternals.mono.dbusmonitor
{
    class Program
    {
        const string indent = "  ";

        static void Main(string[] args)
        {

            Console.WriteLine(".net code example & poc: dbus monitor");
            Console.WriteLine("usage:");
            Console.WriteLine("dbusmonitor [system(default)|session]");

            var systemBus = true;  // look, compiler infered types work!
            if (args.Length >= 1)
                if (args[0].ToUpper().Contains("SESSION"))
                    systemBus = false;

            if (systemBus)
                Console.WriteLine("Monitoring the system bus");
            else
                Console.WriteLine("Monitoring the session bus");

            Console.WriteLine();

            var bus = systemBus ? Bus.System : Bus.Session;
           
            Console.WriteLine("Registered services on bus ID " + bus.UniqueName);

            foreach (var busName in bus.bus.ListNames().Where(n => !n.Contains(":")))  // look, linq works!
                Console.WriteLine(indent + busName);

            bus.AddMatch(MessageFilter.CreateMatchRule(MessageType.Signal));
            bus.AddMatch(MessageFilter.CreateMatchRule(MessageType.MethodReturn));
            bus.AddMatch(MessageFilter.CreateMatchRule(MessageType.Error));
            bus.AddMatch(MessageFilter.CreateMatchRule(MessageType.MethodCall));

            while (true)
            {
                Message msg = bus.ReadMessage();
                if (msg == null)
                    break;
                PrintMessage(msg);
                Console.WriteLine();
            }
        }

        private static void PrintMessage(Message msg)
        {
            Console.WriteLine("Message (" + msg.Header.Endianness + " endian, v" + msg.Header.MajorVersion + "):");
            Console.WriteLine(indent + "Type: " + msg.Header.MessageType);
            Console.WriteLine(indent + "Flags: " + msg.Header.Flags);
            Console.WriteLine(indent + "Serial: " + msg.Header.Serial);

            Console.WriteLine(indent + "Header Fields:");
            foreach (KeyValuePair<FieldCode, object> field in msg.Header.Fields)
                Console.WriteLine(indent + indent + field.Key + ": " + field.Value);

            Console.WriteLine(indent + "Body (" + msg.Header.Length + " bytes):");
            if (msg.Body != null)
            {
                MessageReader reader = new MessageReader(msg);

                try
                {
                    foreach (DType dtype in msg.Signature.GetBuffer())
                    {
                        if (dtype == DType.Invalid)
                            continue;
                        object arg = reader.ReadValue(dtype);
                        Console.WriteLine(indent + indent + "(type:" + dtype + "): " + arg);
                    }
                }
                catch
                {
                    Console.WriteLine(indent + indent + "dbusmonitor is too dumb to decode message body");
                }
            }
        }
    }
}
